@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2019-2022 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2019-2024 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -20,7 +20,9 @@
 #include <functional>
 #include <stack>
 #include <utility>
+#include <wx/artprov.h>
 #include <wx/clipbrd.h>
+#include <wx/statbmp.h>
 
 #ifdef __WXGTK__
 #include <gtk/gtk.h>
@@ -43,6 +45,8 @@ enum {
 	ID_COPY_COMMENT,
 	ID_GOTO,
 	ID_SELECT,
+	ID_FILTER_TEXT,
+	ID_REFRESH_TIMER,
 };
 
 #define MODEL_OFFSET_COLUMN 0
@@ -51,16 +55,34 @@ enum {
 BEGIN_EVENT_TABLE(REHex::CommentTree, wxPanel)
 	EVT_DATAVIEW_ITEM_CONTEXT_MENU(wxID_ANY, REHex::CommentTree::OnContextMenu)
 	EVT_DATAVIEW_ITEM_ACTIVATED(wxID_ANY, REHex::CommentTree::OnActivated)
+	EVT_TIMER(ID_REFRESH_TIMER, REHex::CommentTree::OnRefreshTimer)
+	
+	EVT_TEXT(ID_FILTER_TEXT, REHex::CommentTree::OnFilterTextChange)
 END_EVENT_TABLE()
 
 REHex::CommentTree::CommentTree(wxWindow *parent, SharedDocumentPointer &document, DocumentCtrl *document_ctrl):
 	ToolPanel(parent),
 	document(document),
 	document_ctrl(document_ctrl),
-	historic_max_comment_depth(0)
+	historic_max_comment_depth(0),
+	refresh_running(false),
+	refresh_timer(this, ID_REFRESH_TIMER)
 {
 	model = new CommentTreeModel(this->document, document_ctrl); /* Reference /class/ document pointer! */
 	
+	wxBoxSizer *filter_sizer = new wxBoxSizer(wxHORIZONTAL);
+	
+	filter_textctrl = new wxTextCtrl(this, ID_FILTER_TEXT);
+	filter_textctrl->SetHint("Search text");
+	
+	int filter_height = filter_textctrl->GetSize().GetHeight();
+	
+	wxBitmap find_bitmap = wxArtProvider::GetBitmap(wxART_FIND, wxART_FRAME_ICON, wxSize(filter_height, filter_height));
+	wxStaticBitmap *filter_sbmp = new wxStaticBitmap(this, wxID_ANY, find_bitmap);
+	
+	filter_sizer->Add(filter_sbmp, 0);
+	filter_sizer->Add(filter_textctrl, 1);
+	
 	dvc = new wxDataViewCtrl(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, 0);
 	
 	offset_col = dvc->AppendTextColumn("Offset", MODEL_OFFSET_COLUMN);
@@ -74,10 +96,21 @@ REHex::CommentTree::CommentTree(wxWindow *parent, SharedDocumentPointer &documen
 	/* NOTE: This has to come after AssociateModel, or it will segfault. */
 	offset_col->SetSortOrder(true);
 	
+	spinner = new LoadingSpinner(this, wxID_ANY, wxPoint(0, 0), wxSize(32, 32), wxBORDER_SIMPLE);
+	
 	wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
+	sizer->Add(filter_sizer, 0, wxEXPAND);
 	sizer->Add(dvc, 1, wxEXPAND);
 	SetSizerAndFit(sizer);
 	
+	dvc->Bind(wxEVT_SIZE, [&](wxSizeEvent &event)
+	{
+		reposition_spinner();
+		event.Skip();
+	});
+	
+	reposition_spinner();
+	
 	this->document.auto_cleanup_bind(EV_COMMENT_MODIFIED, &REHex::CommentTree::OnCommentModified, this);
 	
 	refresh_comments();
@@ -116,9 +149,36 @@ wxSize REHex::CommentTree::DoGetBestClientSize() const
 
 void REHex::CommentTree::refresh_comments()
 {
-	model->refresh_comments();
+	if(!refresh_running)
+	{
+		refresh_running = true;
+		spinner->Show();
+		
+		Bind(wxEVT_IDLE, &REHex::CommentTree::OnIdle, this);
+	}
 	
-	#ifdef __WXGTK__
+	bool changed = model->refresh_comments();
+	if(!changed)
+	{
+		#ifdef __APPLE__
+		offset_col->SetWidth(wxCOL_WIDTH_AUTOSIZE); /* Refreshes column width */
+		text_col->SetWidth(wxCOL_WIDTH_AUTOSIZE); /* Refreshes column width */
+		#endif
+		
+		refresh_timer.Stop();
+		Unbind(wxEVT_IDLE, &REHex::CommentTree::OnIdle, this);
+		
+		refresh_running = false;
+		spinner->Hide();
+		return;
+	}
+	
+	/* Schedule a timer to do another update step in case the system is too busy for us to be
+	 * given any idle time slots.
+	*/
+	refresh_timer.Start(MAX_IDLE_WAIT_MS, wxTIMER_ONE_SHOT);
+	
+	#if defined(__WXGTK__)
 	/* wxGTK doesn't account for the expander arrow when using wxCOL_WIDTH_AUTOSIZE, so we need
 	 * to calculate the width ourselves...
 	 *
@@ -161,15 +221,43 @@ void REHex::CommentTree::refresh_comments()
 		offset_col->SetWidth(offset_size.GetWidth() + ((max_comment_depth + 1) * expander_size) + (max_comment_depth * extra_indent));
 	}
 	
-	#else
-	offset_col->SetWidth(wxCOL_WIDTH_AUTOSIZE); /* Refreshes column width */
-	#endif
-	
 	text_col->SetWidth(wxCOL_WIDTH_AUTOSIZE); /* Refreshes column width */
 	
+	#elif defined(__APPLE__)
+	/* wxDataViewColumn::SetWidth() is somewhat expensive on macOS and makes the columns
+	 * twitch around, so we only update the column widths when the maximum depth increases, or
+	 * when a refresh finishes (see above).
+	*/
+	
+	int max_comment_depth = model->get_max_comment_depth();
+	if(max_comment_depth > historic_max_comment_depth)
+	{
+		historic_max_comment_depth = max_comment_depth;
+		
+		offset_col->SetWidth(wxCOL_WIDTH_AUTOSIZE); /* Refreshes column width */
+		text_col->SetWidth(wxCOL_WIDTH_AUTOSIZE); /* Refreshes column width */
+	}
+	
+	#else
+	offset_col->SetWidth(wxCOL_WIDTH_AUTOSIZE); /* Refreshes column width */
+	text_col->SetWidth(wxCOL_WIDTH_AUTOSIZE); /* Refreshes column width */
+	#endif
+	
 	dvc->Refresh();
 }
 
+void REHex::CommentTree::reposition_spinner()
+{
+	wxSize dvc_size = dvc->GetSize();
+	wxPoint dvc_pos = dvc->GetPosition();
+	
+	wxSize spinner_size = spinner->GetSize();
+	
+	spinner->SetPosition(wxPoint(
+		(dvc_pos.x + dvc_size.GetWidth() - spinner_size.GetWidth()),
+		(dvc_pos.y + dvc_size.GetHeight() - spinner_size.GetHeight())));
+}
+
 void REHex::CommentTree::OnCommentModified(wxCommandEvent &event)
 {
 	refresh_comments();
@@ -180,7 +268,7 @@ void REHex::CommentTree::OnContextMenu(wxDataViewEvent &event)
 {
 	assert(document != NULL);
 	
-	const NestedOffsetLengthMapKey *key = CommentTreeModel::dv_item_to_key(event.GetItem());
+	const BitRangeTreeKey *key = CommentTreeModel::dv_item_to_key(event.GetItem());
 	if(key == NULL)
 	{
 		/* Click wasn't over an item. */
@@ -235,13 +323,13 @@ void REHex::CommentTree::OnContextMenu(wxDataViewEvent &event)
 				ClipboardGuard cg;
 				if(cg)
 				{
-					const ByteRangeTree<Document::Comment> &comments = document->get_comments();
-					const ByteRangeTree<Document::Comment>::Node *root_comment = comments.find_node(*key);
+					const BitRangeTree<Document::Comment> &comments = document->get_comments();
+					const BitRangeTree<Document::Comment>::Node *root_comment = comments.find_node(*key);
 					
-					std::list< ByteRangeTree<Document::Comment>::const_iterator > selected_comments;
+					std::list< BitRangeTree<Document::Comment>::const_iterator > selected_comments;
 					
-					std::function<void(const ByteRangeTree<Document::Comment>::Node*)> add_comment;
-					add_comment = [&](const ByteRangeTree<Document::Comment>::Node *comment)
+					std::function<void(const BitRangeTree<Document::Comment>::Node*)> add_comment;
+					add_comment = [&](const BitRangeTree<Document::Comment>::Node *comment)
 					{
 						selected_comments.push_back(comments.find(comment->key));
 						
@@ -273,7 +361,7 @@ void REHex::CommentTree::OnActivated(wxDataViewEvent &event)
 {
 	assert(document != NULL);
 	
-	const NestedOffsetLengthMapKey *key = CommentTreeModel::dv_item_to_key(event.GetItem());
+	const BitRangeTreeKey *key = CommentTreeModel::dv_item_to_key(event.GetItem());
 	assert(key != NULL);
 	
 	document->set_cursor_position(key->offset);
@@ -285,26 +373,59 @@ void REHex::CommentTree::OnActivated(wxDataViewEvent &event)
 	});
 }
 
+void REHex::CommentTree::OnIdle(wxIdleEvent &event)
+{
+	if(refresh_running)
+	{
+		refresh_comments();
+		
+		if(refresh_running)
+		{
+			event.RequestMore();
+		}
+	}
+}
+
+void REHex::CommentTree::OnRefreshTimer(wxTimerEvent &event)
+{
+	if(refresh_running)
+	{
+		refresh_comments();
+	}
+}
+
+void REHex::CommentTree::OnFilterTextChange(wxCommandEvent &event)
+{
+	model->set_filter_text(filter_textctrl->GetValue());
+	refresh_comments();
+}
+
 REHex::CommentTreeModel::CommentTreeModel(SharedDocumentPointer &document, DocumentCtrl *document_ctrl):
 	document(document),
 	document_ctrl(document_ctrl),
-	max_comment_depth(-1) {}
+	max_comment_depth(-1),
+	pending_max_comment_depth(-1) {}
 
-void REHex::CommentTreeModel::refresh_comments()
+#define MAX_CHANGES 100
+
+bool REHex::CommentTreeModel::refresh_comments()
 {
 	PROFILE_BLOCK("REHex::CommentTreeModel::refresh_comments");
 	
-	const ByteRangeTree<Document::Comment> &comments = document->get_comments();
+	const BitRangeTree<Document::Comment> &comments = document->get_comments();
+	unsigned num_changed = 0;
 	
 	/* Erase any comments which no longer exist, or are children of such. */
 	
-	for(auto i = values.begin(); i != values.end();)
+	for(auto i = values.begin(); num_changed < MAX_CHANGES && i != values.end();)
 	{
 		values_elem_t *value = &(*i);
 		
-		if(comments.find(value->first) == comments.end())
+		auto comment = comments.find_node(value->first);
+		if(comment == NULL || !comment_or_child_matches_filter(comment))
 		{
 			i = erase_value(i);
+			++num_changed;
 		}
 		else{
 			++i;
@@ -313,14 +434,17 @@ void REHex::CommentTreeModel::refresh_comments()
 	
 	/* Add any comments which we don't already have registered. */
 	
-	max_comment_depth = -1;
-	
-	std::function<void(const ByteRangeTree<Document::Comment>::Node*, values_elem_t*, int)> add_comment;
-	add_comment = [&](const ByteRangeTree<Document::Comment>::Node *comment, values_elem_t *parent, int depth)
+	std::function<void(const BitRangeTree<Document::Comment>::Node*, values_elem_t*, int)> add_comment;
+	add_comment = [&](const BitRangeTree<Document::Comment>::Node *comment, values_elem_t *parent, int depth)
 	{
-		if(depth > max_comment_depth)
+		if(depth > pending_max_comment_depth)
 		{
-			max_comment_depth = depth;
+			pending_max_comment_depth = depth;
+		}
+		
+		if(!comment_or_child_matches_filter(comment))
+		{
+			return;
 		}
 		
 		auto x = values.emplace(std::make_pair(comment->key, CommentData(parent, comment->value.text)));
@@ -360,26 +484,62 @@ void REHex::CommentTreeModel::refresh_comments()
 				parent->second.children.insert(value);
 			}
 			
-			ItemAdded(wxDataViewItem(parent), wxDataViewItem((void*)(value)));
+			batched_item_added(wxDataViewItem(parent), wxDataViewItem((void*)(value)));
+			++num_changed;
 		}
 		else if(value->second.text.get() != comment->value.text.get())
 		{
 			/* Text has changed. */
 			
 			value->second.text = comment->value.text;
-			ItemChanged(wxDataViewItem((void*)(value)));
+			batched_item_changed(wxDataViewItem((void*)(value)));
+			
+			++num_changed;
 		}
 		
-		for(auto child = comment->get_first_child(); child != NULL; child = child->get_next())
+		for(auto child = comment->get_first_child(); child != NULL && num_changed < MAX_CHANGES; child = child->get_next())
 		{
 			add_comment(child, value, depth + 1);
 		}
 	};
 	
-	for(auto comment = comments.first_root_node(); comment != NULL; comment = comment->get_next())
+	for(auto comment = comments.first_root_node(); comment != NULL && num_changed < MAX_CHANGES; comment = comment->get_next())
 	{
 		add_comment(comment, NULL, 0);
 	}
+	
+	if(num_changed < MAX_CHANGES)
+	{
+		max_comment_depth = pending_max_comment_depth;
+		pending_max_comment_depth = -1;
+	}
+	
+	batched_item_flush();
+	
+	return num_changed > 0;
+}
+
+bool REHex::CommentTreeModel::comment_or_child_matches_filter(const BitRangeTree<Document::Comment>::Node *comment)
+{
+	if(filter_text.empty())
+	{
+		return true;
+	}
+	
+	if(comment->value.text.get()->Find(filter_text) != wxNOT_FOUND)
+	{
+		return true;
+	}
+	
+	for(auto child = comment->get_first_child(); child != NULL; child = child->get_next())
+	{
+		if(comment_or_child_matches_filter(child))
+		{
+			return true;
+		}
+	}
+	
+	return false;
 }
 
 int REHex::CommentTreeModel::get_max_comment_depth() const
@@ -387,12 +547,22 @@ int REHex::CommentTreeModel::get_max_comment_depth() const
 	return max_comment_depth;
 }
 
-const REHex::NestedOffsetLengthMapKey *REHex::CommentTreeModel::dv_item_to_key(const wxDataViewItem &item)
+const REHex::BitRangeTreeKey *REHex::CommentTreeModel::dv_item_to_key(const wxDataViewItem &item)
 {
-	return (const NestedOffsetLengthMapKey*)(item.GetID());
+	return (const BitRangeTreeKey*)(item.GetID());
 }
 
-std::map<REHex::NestedOffsetLengthMapKey, REHex::CommentTreeModel::CommentData>::iterator REHex::CommentTreeModel::erase_value(std::map<REHex::NestedOffsetLengthMapKey, REHex::CommentTreeModel::CommentData>::iterator value_i)
+void REHex::CommentTreeModel::set_filter_text(const wxString &filter_text)
+{
+	this->filter_text = filter_text;
+}
+
+wxString REHex::CommentTreeModel::get_filter_text() const
+{
+	return filter_text;
+}
+
+std::map<REHex::BitRangeTreeKey, REHex::CommentTreeModel::CommentData>::iterator REHex::CommentTreeModel::erase_value(std::map<REHex::BitRangeTreeKey, REHex::CommentTreeModel::CommentData>::iterator value_i)
 {
 	values_elem_t *value  = &(*value_i);
 	values_elem_t *parent = value->second.parent;
@@ -403,6 +573,13 @@ std::map<REHex::NestedOffsetLengthMapKey, REHex::CommentTreeModel::CommentData>:
 		erase_value(values.find(child->first));
 	}
 	
+	#ifdef COMMENTTREEMODEL_BATCH_MODEL_UPDATES
+	if(!accumulated_items_to_add.IsEmpty() || !accumulated_items_to_change.IsEmpty())
+	{
+		batched_item_flush();
+	}
+	#endif
+	
 	bool parent_became_empty = false;
 	if(parent == NULL)
 	{
@@ -415,7 +592,7 @@ std::map<REHex::NestedOffsetLengthMapKey, REHex::CommentTreeModel::CommentData>:
 	
 	auto next_value_i = values.erase(value_i);
 	
-	ItemDeleted(wxDataViewItem(parent), wxDataViewItem(value));
+	batched_item_deleted(wxDataViewItem(parent), wxDataViewItem(value));
 	
 	if(parent_became_empty)
 	{
@@ -439,16 +616,23 @@ void REHex::CommentTreeModel::re_add_item(values_elem_t *value, bool as_containe
 	*/
 	
 	std::shared_ptr<const wxString> placeholder_text(new wxString(""));
-	values_elem_t placeholder(NestedOffsetLengthMapKey(-1, 0), CommentData(parent, placeholder_text));
+	values_elem_t placeholder(BitRangeTreeKey(BitOffset(-1, 0), BitOffset(0, 0)), CommentData(parent, placeholder_text));
 	bool added_placeholder = false;
 	
 	if(parent != NULL && parent->second.children.size() == 1U)
 	{
 		parent->second.children.insert(&placeholder);
-		ItemAdded(wxDataViewItem(parent), wxDataViewItem(&placeholder));
+		batched_item_added(wxDataViewItem(parent), wxDataViewItem(&placeholder));
 		added_placeholder = true;
 	}
 	
+	#ifdef COMMENTTREEMODEL_BATCH_MODEL_UPDATES
+	if(!accumulated_items_to_add.IsEmpty() || !accumulated_items_to_change.IsEmpty())
+	{
+		batched_item_flush();
+	}
+	#endif
+	
 	if(parent != NULL)
 	{
 		parent->second.children.erase(value);
@@ -457,7 +641,7 @@ void REHex::CommentTreeModel::re_add_item(values_elem_t *value, bool as_containe
 		root.erase(value);
 	}
 	
-	ItemDeleted(wxDataViewItem(parent), wxDataViewItem(value));
+	batched_item_deleted(wxDataViewItem(parent), wxDataViewItem(value));
 	
 	value->second.is_container = as_container;
 	
@@ -469,15 +653,105 @@ void REHex::CommentTreeModel::re_add_item(values_elem_t *value, bool as_containe
 		root.insert(value);
 	}
 	
-	ItemAdded(wxDataViewItem(parent), wxDataViewItem(value));
+	batched_item_added(wxDataViewItem(parent), wxDataViewItem(value));
 	
 	if(added_placeholder)
 	{
 		parent->second.children.erase(&placeholder);
-		ItemDeleted(wxDataViewItem(parent), wxDataViewItem(&placeholder));
+		batched_item_deleted(wxDataViewItem(parent), wxDataViewItem(&placeholder));
 	}
 }
 
+void REHex::CommentTreeModel::batched_item_added(const wxDataViewItem &parent, const wxDataViewItem &item)
+{
+	#ifdef COMMENTTREEMODEL_BATCH_MODEL_UPDATES
+	if(accumulated_items_to_add.GetCount() >= COMMENTREEEMODEL_MAX_BATCHED_UPDATES
+		|| !accumulated_items_to_delete.IsEmpty()
+		|| !accumulated_items_to_change.IsEmpty()
+		|| accumulated_items_parent != parent)
+	{
+		batched_item_flush();
+	}
+	
+	accumulated_items_parent = parent;
+	accumulated_items_to_add.Add(item);
+	
+	#else
+	ItemAdded(parent, item);
+	
+	#endif
+}
+
+void REHex::CommentTreeModel::batched_item_deleted(const wxDataViewItem &parent, const wxDataViewItem &item)
+{
+	#ifdef COMMENTTREEMODEL_BATCH_MODEL_UPDATES
+	if(accumulated_items_to_delete.GetCount() >= COMMENTREEEMODEL_MAX_BATCHED_UPDATES
+		|| !accumulated_items_to_add.IsEmpty()
+		|| !accumulated_items_to_change.IsEmpty()
+		|| accumulated_items_parent != parent)
+	{
+		batched_item_flush();
+	}
+	
+	accumulated_items_parent = parent;
+	accumulated_items_to_delete.Add(item);
+	
+	#else
+	ItemDeleted(parent, item);
+	
+	#endif
+}
+
+void REHex::CommentTreeModel::batched_item_changed(const wxDataViewItem &item)
+{
+	#ifdef COMMENTTREEMODEL_BATCH_MODEL_UPDATES
+	if(accumulated_items_to_change.GetCount() >= COMMENTREEEMODEL_MAX_BATCHED_UPDATES
+		|| !accumulated_items_to_add.IsEmpty()
+		|| !accumulated_items_to_delete.IsEmpty())
+	{
+		batched_item_flush();
+	}
+	
+	accumulated_items_to_change.Add(item);
+	
+	#else
+	ItemChanged(item);
+	
+	#endif
+}
+
+void REHex::CommentTreeModel::batched_item_flush()
+{
+	#ifdef COMMENTTREEMODEL_BATCH_MODEL_UPDATES
+	
+	if(!accumulated_items_to_add.IsEmpty())
+	{
+		assert(accumulated_items_to_delete.IsEmpty());
+		assert(accumulated_items_to_change.IsEmpty());
+		
+		ItemsAdded(accumulated_items_parent, accumulated_items_to_add);
+		accumulated_items_to_add.Empty();
+	}
+	else if(!accumulated_items_to_delete.IsEmpty())
+	{
+		assert(accumulated_items_to_add.IsEmpty());
+		assert(accumulated_items_to_change.IsEmpty());
+		
+		ItemsDeleted(accumulated_items_parent, accumulated_items_to_delete);
+		accumulated_items_to_delete.Empty();
+	}
+	else if(!accumulated_items_to_change.IsEmpty())
+	{
+		assert(accumulated_items_to_add.IsEmpty());
+		assert(accumulated_items_to_delete.IsEmpty());
+		
+		ItemsChanged(accumulated_items_to_change);
+		accumulated_items_to_change.Empty();
+	}
+	
+	#endif
+}
+
 int REHex::CommentTreeModel::Compare(const wxDataViewItem &item1, const wxDataViewItem &item2, unsigned int column, bool ascending) const
 {
 	assert(column == MODEL_OFFSET_COLUMN);
@@ -485,8 +759,8 @@ int REHex::CommentTreeModel::Compare(const wxDataViewItem &item1, const wxDataVi
 	assert(item1.IsOk());
 	assert(item2.IsOk());
 	
-	const NestedOffsetLengthMapKey *key1 = (const NestedOffsetLengthMapKey*)(item1.GetID());
-	const NestedOffsetLengthMapKey *key2 = (const NestedOffsetLengthMapKey*)(item2.GetID());
+	const BitRangeTreeKey *key1 = (const BitRangeTreeKey*)(item1.GetID());
+	const BitRangeTreeKey *key2 = (const BitRangeTreeKey*)(item2.GetID());
 	
 	int result;
 	if(key1->offset < key2->offset)
