@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2020-2021 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2020-2024 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -40,18 +40,17 @@ REHex::FillRangeDialog::FillRangeDialog(wxWindow *parent, Document &document, Do
 	char initial_to[64]   = "";
 	char initial_len[64]  = "";
 	
-	std::pair<off_t, off_t> selection = document_ctrl.get_selection_linear();
-	off_t selection_off    = selection.first;
-	off_t selection_length = selection.second;
+	BitOffset selection_off, selection_length;
+	std::tie(selection_off, selection_length) = document_ctrl.get_selection_linear();
 	
-	if(selection_length > 0)
+	if(selection_length > BitOffset::ZERO && selection_off.byte_aligned() && selection_length.byte_aligned())
 	{
-		snprintf(initial_from, sizeof(initial_from), "0x%08llX", (long long unsigned)(selection_off));
-		snprintf(initial_to,   sizeof(initial_to),   "0x%08llX", (long long unsigned)(selection_off + selection_length - 1));
-		snprintf(initial_len,  sizeof(initial_len),  "0x%08llX", (long long unsigned)(selection_length));
+		snprintf(initial_from, sizeof(initial_from), "0x%08llX", (long long unsigned)(selection_off.byte()));
+		snprintf(initial_to,   sizeof(initial_to),   "0x%08llX", (long long unsigned)((selection_off + selection_length - 1).byte()));
+		snprintf(initial_len,  sizeof(initial_len),  "0x%08llX", (long long unsigned)(selection_length.byte()));
 	}
 	else{
-		off_t cursor_pos = document_ctrl.get_cursor_position();
+		off_t cursor_pos = document_ctrl.get_cursor_position().byte();
 		snprintf(initial_from, sizeof(initial_from), "0x%08llX", (long long unsigned)(cursor_pos));
 	}
 	
@@ -67,6 +66,21 @@ REHex::FillRangeDialog::FillRangeDialog(wxWindow *parent, Document &document, Do
 		topsizer->Add(data_sizer, 0, wxEXPAND | wxALL, 10);
 	}
 	
+	{
+		wxBoxSizer *mode_sizer = new wxBoxSizer(wxHORIZONTAL);
+		
+		overwrite_mode = new wxRadioButton(this, wxID_ANY, "Overwrite data", wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
+		mode_sizer->Add(overwrite_mode, 0, wxALIGN_CENTER_VERTICAL);
+		
+		insert_mode = new wxRadioButton(this, wxID_ANY, "Insert data");
+		mode_sizer->Add(insert_mode, 1, wxALIGN_CENTER_VERTICAL | wxLEFT, 10);
+		
+		topsizer->Add(mode_sizer, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, 10);
+		
+		/* Default to overwrite mode. */
+		overwrite_mode->SetValue(true);
+	}
+	
 	topsizer->Add(
 		new wxStaticText(this, wxID_ANY, "Data will be truncated or repeated as\n"
 		                                 "necessary to fill the entire range."),
@@ -83,18 +97,26 @@ REHex::FillRangeDialog::FillRangeDialog(wxWindow *parent, Document &document, Do
 		range_from = new NumericTextCtrl(this, wxID_ANY, initial_from);
 		from_sizer->Add(range_from, 0, wxALIGN_CENTER_VERTICAL | wxLEFT, 10);
 		
+		wxSize initial_size = range_from->GetSize();
+		wxSize text_size = range_from->GetTextExtent("0x0000000000000000+0b");
+		range_from->SetMinSize(wxSize(((float)(text_size.GetWidth()) * 1.2f), initial_size.GetHeight()));
+		
 		range_sizer->Add(from_sizer, 0, wxEXPAND | wxALL, 10);
 	}
 	
 	{
 		wxBoxSizer *to_sizer = new wxBoxSizer(wxHORIZONTAL);
 		
-		range_to_enable = new wxRadioButton(this, wxID_ANY, "To offset");
+		range_to_enable = new wxRadioButton(this, wxID_ANY, "To offset", wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
 		to_sizer->Add(range_to_enable, 1, wxALIGN_CENTER_VERTICAL);
 		
 		range_to = new NumericTextCtrl(this, wxID_ANY, initial_to);
 		to_sizer->Add(range_to, 0, wxALIGN_CENTER_VERTICAL | wxLEFT, 10);
 		
+		wxSize initial_size = range_to->GetSize();
+		wxSize text_size = range_to->GetTextExtent("0x0000000000000000+0b");
+		range_to->SetMinSize(wxSize(((float)(text_size.GetWidth()) * 1.2f), initial_size.GetHeight()));
+		
 		range_sizer->Add(to_sizer, 0, wxEXPAND | wxLEFT | wxRIGHT, 10);
 		
 		/* Default to "To offset" */
@@ -110,6 +132,10 @@ REHex::FillRangeDialog::FillRangeDialog(wxWindow *parent, Document &document, Do
 		range_len = new NumericTextCtrl(this, wxID_ANY, initial_len);
 		len_sizer->Add(range_len, 0, wxALIGN_CENTER_VERTICAL | wxLEFT, 10);
 		
+		wxSize initial_size = range_len->GetSize();
+		wxSize text_size = range_len->GetTextExtent("0x0000000000000000+0b");
+		range_len->SetMinSize(wxSize(((float)(text_size.GetWidth()) * 1.2f), initial_size.GetHeight()));
+		
 		range_sizer->Add(len_sizer, 0, wxEXPAND | wxLEFT | wxRIGHT, 10);
 	}
 	
@@ -160,11 +186,19 @@ void REHex::FillRangeDialog::OnOK(wxCommandEvent &event)
 		return;
 	}
 	
+	bool insert_mode_selected = insert_mode->GetValue();
+	
 	off_t doc_length = document.buffer_length();
 	off_t selection_off, selection_length;
 	
 	try {
-		selection_off = range_from->GetValue<off_t>(0, (doc_length - 1));
+		if(insert_mode_selected)
+		{
+			selection_off = range_from->GetValue<off_t>(0, doc_length);
+		}
+		else{
+			selection_off = range_from->GetValue<off_t>(0, (doc_length - 1));
+		}
 	}
 	catch(const NumericTextCtrl::InputError &e)
 	{
@@ -178,8 +212,15 @@ void REHex::FillRangeDialog::OnOK(wxCommandEvent &event)
 	if(range_to_enable->GetValue())
 	{
 		try {
-			off_t selection_to = range_to->GetValue<off_t>(selection_off, (doc_length - 1));
-			selection_length = (selection_to - selection_off) + 1;
+			if(insert_mode_selected)
+			{
+				off_t selection_to = range_to->GetValue<off_t>(selection_off);
+				selection_length = (selection_to - selection_off) + 1;
+			}
+			else{
+				off_t selection_to = range_to->GetValue<off_t>(selection_off, (doc_length - 1));
+				selection_length = (selection_to - selection_off) + 1;
+			}
 		}
 		catch(const NumericTextCtrl::InputError &e)
 		{
@@ -192,7 +233,13 @@ void REHex::FillRangeDialog::OnOK(wxCommandEvent &event)
 	else if(range_len_enable->GetValue())
 	{
 		try {
-			selection_length = range_len->GetValue<off_t>(0, (doc_length - selection_off));
+			if(insert_mode_selected)
+			{
+				selection_length = range_len->GetValue<off_t>(0);
+			}
+			else{
+				selection_length = range_len->GetValue<off_t>(0, (doc_length - selection_off));
+			}
 		}
 		catch(const NumericTextCtrl::InputError &e)
 		{
@@ -229,8 +276,15 @@ void REHex::FillRangeDialog::OnOK(wxCommandEvent &event)
 		return;
 	}
 	
-	document.overwrite_data(selection_off, data.data(), data.size(),
-		-1, Document::CSTATE_CURRENT, "fill range");
+	if(insert_mode_selected)
+	{
+		document.insert_data(selection_off, data.data(), data.size(),
+			-1, Document::CSTATE_CURRENT, "fill range");
+	}
+	else{
+		document.overwrite_data(selection_off, data.data(), data.size(),
+			-1, Document::CSTATE_CURRENT, "fill range");
+	}
 	
 	Close();
 }
