@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2018-2022 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2018-2024 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -62,7 +62,7 @@ REHex::DecodePanel::DecodePanel(wxWindow *parent, SharedDocumentPointer &documen
 	pgrid = new wxPropertyGrid(this, wxID_ANY, wxDefaultPosition, wxDefaultSize,
 		wxPG_STATIC_SPLITTER);
 	
-	wxGetApp().Bind(PALETTE_CHANGED, [this](wxCommandEvent &event) { set_pgrid_colours(); });
+	wxGetApp().Bind(PALETTE_CHANGED, &REHex::DecodePanel::OnColourPaletteChanged, this);
 	set_pgrid_colours();
 	
 	pgrid->Append(c8 = new wxPropertyCategory("8 bit integer"));
@@ -137,6 +137,11 @@ REHex::DecodePanel::DecodePanel(wxWindow *parent, SharedDocumentPointer &documen
 	update();
 }
 
+REHex::DecodePanel::~DecodePanel()
+{
+	wxGetApp().Unbind(PALETTE_CHANGED, &REHex::DecodePanel::OnColourPaletteChanged, this);
+}
+
 std::string REHex::DecodePanel::name() const
 {
 	return "DecodePanel";
@@ -393,8 +398,8 @@ void REHex::DecodePanel::OnPropertyGridSelected(wxPropertyGridEvent &event)
 	
 	if(size > 0 && document_ctrl != NULL)
 	{
-		off_t cursor_position = document->get_cursor_position();
-		document_ctrl->set_selection_raw(cursor_position, (cursor_position + size - 1));
+		BitOffset cursor_position = document->get_cursor_position();
+		document_ctrl->set_selection_raw(cursor_position, (cursor_position + BitOffset(size, 0) - BitOffset(0, 1)));
 	}
 }
 
@@ -411,6 +416,12 @@ void REHex::DecodePanel::OnSize(wxSizeEvent &event)
 	event.Skip();
 }
 
+void REHex::DecodePanel::OnColourPaletteChanged(wxCommandEvent &event)
+{
+	set_pgrid_colours();
+	event.Skip();
+}
+
 template<typename T, int base, T (*htoX)(T)> void REHex::DecodePanel::OnSignedValue(wxStringProperty *property)
 {
 	static_assert(std::numeric_limits<T>::is_integer, "OnSignedValue() instantiated with non-integer type");
