@@ -10,9 +10,18 @@
 
 #define strncasecmp _strnicmp
 
+#define S_ISREG(mode) ((mode & _S_IFMT) == _S_IFREG)
+#define S_ISBLK(mode) (false)
+
 // To ensure we are the only one defining it, define: _OFF_T_DEFINED
 // See msvc/rehex.props
 typedef __int64 _off_t;
 typedef _off_t off_t;
 
+#ifdef _WIN64
+typedef __int64 ssize_t;
+#else
+typedef int ssize_t;
+#endif
+
 #endif // _MSC_VER
