@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2018-2021 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2018-2024 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -17,6 +17,7 @@
 
 #include "platform.hpp"
 #include <assert.h>
+#include <cmath>
 #include <functional>
 #include <stdlib.h>
 #include <string.h>
@@ -36,7 +37,7 @@
 /* This MUST come after the wxWidgets headers have been included, else we pull in windows.h BEFORE the wxWidgets
  * headers when building on Windows and this causes unicode-flavoured pointer conversion errors.
 */
-#include <portable_endian.h>
+#include "endian_conv.hpp"
 
 enum {
 	ID_FIND_NEXT = 1,
@@ -69,6 +70,7 @@ BEGIN_EVENT_TABLE(REHex::Search, wxDialog)
 	
 	EVT_BUTTON(ID_FIND_NEXT, REHex::Search::OnFindNext)
 	EVT_BUTTON(ID_FIND_PREV, REHex::Search::OnFindPrev)
+	EVT_TEXT_ENTER(wxID_ANY, REHex::Search::OnTextEnter)
 	EVT_BUTTON(wxID_CANCEL, REHex::Search::OnCancel)
 	EVT_TIMER(ID_TIMER, REHex::Search::OnTimer)
 END_EVENT_TABLE()
@@ -76,6 +78,7 @@ END_EVENT_TABLE()
 REHex::Search::Search(wxWindow *parent, SharedDocumentPointer &doc, const char *title):
 	wxDialog(parent, wxID_ANY, title),
 	doc(doc), range_begin(0), range_end(-1), align_to(1), align_from(0), match_found_at(-1), running(false),
+	search_end_focus(NULL),
 	timer(this, ID_TIMER),
 	auto_close(false),
 	auto_wrap(false),
@@ -279,7 +282,7 @@ void REHex::Search::OnFindNext(wxCommandEvent &event)
 {
 	if(read_base_window_controls() && read_window_controls())
 	{
-		begin_search((doc->get_cursor_position() + 1), range_end, SearchDirection::FORWARDS);
+		begin_search((doc->get_cursor_position().byte() + 1), range_end, SearchDirection::FORWARDS);
 	}
 }
 
@@ -287,7 +290,29 @@ void REHex::Search::OnFindPrev(wxCommandEvent &event)
 {
 	if(read_base_window_controls() && read_window_controls())
 	{
-		begin_search(range_begin, doc->get_cursor_position(), SearchDirection::BACKWARDS);
+		begin_search(range_begin, doc->get_cursor_position().byte(), SearchDirection::BACKWARDS);
+	}
+}
+
+void REHex::Search::OnTextEnter(wxCommandEvent &event)
+{
+	/* The search progress dialog steals focus from whatever text control the user just pressed
+	 * enter in, we stash the control and current selection (includes cursor position) so we can
+	 * restore it when the search is finished.
+	*/
+	
+	wxTextCtrl *control = dynamic_cast<wxTextCtrl*>(event.GetEventObject());
+	assert(control != NULL);
+	
+	search_end_focus = control;
+	control->GetSelection(&search_end_focus_from, &search_end_focus_to);
+	
+	if(wxGetKeyState(WXK_SHIFT))
+	{
+		OnFindPrev(event);
+	}
+	else{
+		OnFindNext(event);
 	}
 }
 
@@ -302,6 +327,13 @@ void REHex::Search::OnTimer(wxTimerEvent &event)
 	{
 		end_search();
 		
+		if(search_end_focus != NULL)
+		{
+			search_end_focus->SetFocus();
+			search_end_focus->SetSelection(search_end_focus_from, search_end_focus_to);
+			search_end_focus = NULL;
+		}
+		
 		if(auto_close)
 		{
 			Destroy();
@@ -316,7 +348,7 @@ void REHex::Search::OnTimer(wxTimerEvent &event)
 		
 		if(match_found_at >= 0)
 		{
-			doc->set_cursor_position(match_found_at);
+			doc->set_cursor_position(BitOffset(match_found_at));
 		}
 		else{
 			size_t compare_size = test_max_window();
@@ -354,6 +386,13 @@ void REHex::Search::OnTimer(wxTimerEvent &event)
 			}
 		}
 		
+		if(search_end_focus != NULL)
+		{
+			search_end_focus->SetFocus();
+			search_end_focus->SetSelection(search_end_focus_from, search_end_focus_to);
+			search_end_focus = NULL;
+		}
+		
 		if(auto_close)
 		{
 			Destroy();
@@ -661,7 +700,7 @@ void REHex::Search::Text::setup_window_controls(wxWindow *parent, wxSizer *sizer
 		
 		text_sizer->Add(new wxStaticText(parent, wxID_ANY, "Text: "), 0, wxALIGN_CENTER_VERTICAL);
 		
-		search_for_tc = new wxTextCtrl(parent, wxID_ANY, "");
+		search_for_tc = new wxTextCtrl(parent, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER);
 		text_sizer->Add(search_for_tc, 1);
 		
 		sizer->Add(text_sizer, 0, wxTOP | wxLEFT | wxRIGHT | wxEXPAND, 10);
@@ -788,7 +827,7 @@ void REHex::Search::ByteSequence::setup_window_controls(wxWindow *parent, wxSize
 		
 		text_sizer->Add(new wxStaticText(parent, wxID_ANY, "Data: "), 0, wxALIGN_CENTER_VERTICAL);
 		
-		search_for_tc = new wxTextCtrl(parent, wxID_ANY, "");
+		search_for_tc = new wxTextCtrl(parent, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER);
 		text_sizer->Add(search_for_tc, 1);
 		
 		sizer->Add(text_sizer, 0, wxTOP | wxLEFT | wxRIGHT | wxEXPAND, 10);
@@ -834,7 +873,7 @@ REHex::Search::Value::~Value()
 	}
 }
 
-void REHex::Search::Value::configure(const std::string &value, unsigned formats)
+void REHex::Search::Value::configure(const std::string &value, unsigned formats, const std::string &epsilon)
 {
 	search_for_tc->SetValue(value);
 	
@@ -855,6 +894,10 @@ void REHex::Search::Value::configure(const std::string &value, unsigned formats)
 	i16_cb->SetValue(!!(formats & FMT_I16));
 	i32_cb->SetValue(!!(formats & FMT_I32));
 	i64_cb->SetValue(!!(formats & FMT_I64));
+	f32_cb->SetValue(!!(formats & FMT_F32));
+	f64_cb->SetValue(!!(formats & FMT_F64));
+	
+	epsilon_tc->SetValue(epsilon);
 	
 	read_window_controls();
 }
@@ -869,6 +912,84 @@ bool REHex::Search::Value::test(const void *data, size_t data_size)
 		}
 	}
 	
+	if(f32_enabled && data_size >= sizeof(float))
+	{
+		if(be_enabled)
+		{
+			float f = beXXXtoh_p<float>(data);
+			
+			if(isnan(f) && isnan(f32_value))
+			{
+				return true;
+			}
+			else if(isinf(f) && isinf(f32_value) && f == f32_value)
+			{
+				return true;
+			}
+			else if(std::isfinite(f) && std::isfinite(f32_value) && fabsf(f - f32_value) <= f32_epsilon)
+			{
+				return true;
+			}
+		}
+		
+		if(le_enabled)
+		{
+			float f = leXXXtoh_p<float>(data);
+			
+			if(isnan(f) && isnan(f32_value))
+			{
+				return true;
+			}
+			else if(isinf(f) && isinf(f32_value) && f == f32_value)
+			{
+				return true;
+			}
+			else if(std::isfinite(f) && std::isfinite(f32_value) && fabsf(f - f32_value) <= f32_epsilon)
+			{
+				return true;
+			}
+		}
+	}
+	
+	if(f64_enabled && data_size >= sizeof(double))
+	{
+		if(be_enabled)
+		{
+			double f = beXXXtoh_p<double>(data);
+			
+			if(isnan(f) && isnan(f64_value))
+			{
+				return true;
+			}
+			else if(isinf(f) && isinf(f64_value) && f == f64_value)
+			{
+				return true;
+			}
+			else if(std::isfinite(f) && std::isfinite(f64_value) && fabs(f - f64_value) <= f64_epsilon)
+			{
+				return true;
+			}
+		}
+		
+		if(le_enabled)
+		{
+			double f = leXXXtoh_p<double>(data);
+			
+			if(isnan(f) && isnan(f64_value))
+			{
+				return true;
+			}
+			else if(isinf(f) && isinf(f64_value) && f == f64_value)
+			{
+				return true;
+			}
+			else if(std::isfinite(f) && std::isfinite(f64_value) && fabs(f - f64_value) <= f64_epsilon)
+			{
+				return true;
+			}
+		}
+	}
+	
 	return false;
 }
 
@@ -881,6 +1002,16 @@ size_t REHex::Search::Value::test_max_window()
 		search_for_max = std::max(search_for_max, i->size());
 	}
 	
+	if(f32_enabled)
+	{
+		search_for_max = std::max(search_for_max, sizeof(float));
+	}
+	
+	if(f64_enabled)
+	{
+		search_for_max = std::max(search_for_max, sizeof(double));
+	}
+	
 	return search_for_max;
 }
 
@@ -891,7 +1022,7 @@ void REHex::Search::Value::setup_window_controls(wxWindow *parent, wxSizer *size
 		
 		text_sizer->Add(new wxStaticText(parent, wxID_ANY, "Value: "), 0, wxALIGN_CENTER_VERTICAL);
 		
-		search_for_tc = new NumericTextCtrl(parent, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, 0);
+		search_for_tc = new NumericTextCtrl(parent, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER);
 		text_sizer->Add(search_for_tc, 1);
 		
 		search_for_tc->Bind(wxEVT_TEXT, &REHex::Search::Value::OnText, this);
@@ -902,7 +1033,7 @@ void REHex::Search::Value::setup_window_controls(wxWindow *parent, wxSizer *size
 	{
 		wxStaticBoxSizer *sz = new wxStaticBoxSizer(wxVERTICAL, parent, "Value formats");
 		
-		wxBoxSizer *sz1 = new wxBoxSizer(wxHORIZONTAL);
+		wxGridSizer *sz1 = new wxGridSizer(4);
 		sz->Add(sz1, 0, wxTOP | wxBOTTOM, 5);
 		
 		i8_cb = new wxCheckBox(sz->GetStaticBox(), wxID_ANY, "8-bit integer");
@@ -921,6 +1052,17 @@ void REHex::Search::Value::setup_window_controls(wxWindow *parent, wxSizer *size
 		i64_cb->SetValue(true);
 		sz1->Add(i64_cb, 0, wxLEFT | wxRIGHT, 5);
 		
+		sz1->AddSpacer(1);
+		sz1->AddSpacer(1);
+		
+		f32_cb = new wxCheckBox(sz->GetStaticBox(), wxID_ANY, "32-bit float");
+		f32_cb->SetValue(true);
+		sz1->Add(f32_cb, 0, wxLEFT, 5);
+		
+		f64_cb = new wxCheckBox(sz->GetStaticBox(), wxID_ANY, "64-bit float");
+		f64_cb->SetValue(true);
+		sz1->Add(f64_cb, 0, wxLEFT | wxRIGHT, 5);
+		
 		wxStaticLine *sl1 = new wxStaticLine(sz->GetStaticBox(), wxID_ANY, wxDefaultPosition, wxDefaultSize, wxLI_HORIZONTAL);
 		sz->Add(sl1, 0, wxEXPAND | wxLEFT | wxRIGHT, 5);
 		
@@ -937,6 +1079,18 @@ void REHex::Search::Value::setup_window_controls(wxWindow *parent, wxSizer *size
 		e_either->SetValue(true);
 		sz2->Add(e_either, 0, wxLEFT | wxRIGHT, 5);
 		
+		wxStaticLine *sl2 = new wxStaticLine(sz->GetStaticBox(), wxID_ANY, wxDefaultPosition, wxDefaultSize, wxLI_HORIZONTAL);
+		sz->Add(sl2, 0, wxEXPAND | wxLEFT | wxRIGHT, 5);
+		
+		wxBoxSizer *sz3 = new wxBoxSizer(wxHORIZONTAL);
+		sz->Add(sz3, 0, wxTOP | wxBOTTOM, 5);
+		
+		sz3->Add(new wxStaticText(sz->GetStaticBox(), wxID_ANY, "Floating point epsilon:"), 0, wxLEFT | wxALIGN_CENTER_VERTICAL, 5);
+		
+		epsilon_tc = new NumericTextCtrl(sz->GetStaticBox(), wxID_ANY, "0", wxDefaultPosition, wxDefaultSize, 0);
+		epsilon_tc->SetToolTip("Tolerance when comparing floating point numbers");
+		sz3->Add(epsilon_tc, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
+		
 		sizer->Add(sz, 0, wxTOP | wxLEFT | wxRIGHT | wxEXPAND, 10);
 	}
 }
@@ -987,6 +1141,9 @@ bool REHex::Search::Value::read_window_controls()
 {
 	search_for.clear();
 	
+	le_enabled = e_little->GetValue() || e_either->GetValue();
+	be_enabled = e_big->GetValue() || e_either->GetValue();
+	
 	if(i8_cb->GetValue())
 	{
 		try {
@@ -1008,7 +1165,29 @@ bool REHex::Search::Value::read_window_controls()
 	INTEGER_TYPE_SIZE(32);
 	INTEGER_TYPE_SIZE(64);
 	
-	if(search_for.empty())
+	f32_enabled = false;
+	if(f32_cb->GetValue())
+	{
+		try {
+			f32_value = parse_float(search_for_tc->GetStringValue().ToStdString());
+			f32_epsilon = parse_float(epsilon_tc->GetStringValue().ToStdString());
+			f32_enabled = true;
+		}
+		catch(const ParseError&) {}
+	}
+	
+	f64_enabled = false;
+	if(f64_cb->GetValue())
+	{
+		try {
+			f64_value = parse_double(search_for_tc->GetStringValue().ToStdString());
+			f64_epsilon = parse_double(epsilon_tc->GetStringValue().ToStdString());
+			f64_enabled = true;
+		}
+		catch(const ParseError&) {}
+	}
+	
+	if(search_for.empty() && !f32_enabled && !f64_enabled)
 	{
 		wxMessageBox("Please enter a valid value to search for", "Error", (wxOK | wxICON_EXCLAMATION | wxCENTRE), this);
 		return false;
@@ -1050,4 +1229,14 @@ void REHex::Search::Value::OnText(wxCommandEvent &event)
 	
 	try { search_for_tc->GetValue<uint64_t>(); i64_cb->Enable(); }
 	catch(const REHex::NumericTextCtrl::InputError &) {}
+	
+	f32_cb->Disable();
+	
+	try { parse_float(search_for_tc->GetStringValue().ToStdString()); f32_cb->Enable(); }
+	catch(const ParseError&) {}
+	
+	f64_cb->Disable();
+	
+	try { parse_double(search_for_tc->GetStringValue().ToStdString()); f64_cb->Enable(); }
+	catch(const ParseError&) {}
 }
