@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2018-2022 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2018-2024 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -18,12 +18,15 @@
 #ifndef REHEX_UTIL_HPP
 #define REHEX_UTIL_HPP
 
+#include <jansson.h>
 #include <iterator>
 #include <stdexcept>
 #include <string>
 #include <vector>
 #include <wx/window.h>
 
+#include "BitOffset.hpp"
+
 namespace REHex {
 	class ParseError: public std::runtime_error
 	{
@@ -31,6 +34,24 @@ namespace REHex {
 			ParseError(const char *what);
 	};
 	
+	class ParseErrorFormat: public ParseError
+	{
+		public:
+			ParseErrorFormat(): ParseError("Number is not of a known format") {}
+	};
+	
+	class ParseErrorRange: public ParseError
+	{
+		public:
+			ParseErrorRange(): ParseError("Number is out of range") {}
+	};
+	
+	class ParseErrorEmpty: public ParseError
+	{
+		public:
+			ParseErrorEmpty(): ParseError("No number provided") {}
+	};
+	
 	/**
 	 * @brief RAII-style access to the clipboard.
 	 *
@@ -70,6 +91,9 @@ namespace REHex {
 	std::vector<unsigned char> parse_hex_string(const std::string &hex_string);
 	unsigned char parse_ascii_nibble(char c);
 	
+	float parse_float(const std::string &s);
+	double parse_double(const std::string &s);
+	
 	void file_manager_show_file(const std::string &filename);
 	
 	enum OffsetBase {
@@ -81,6 +105,7 @@ namespace REHex {
 	};
 	
 	std::string format_offset(off_t offset, OffsetBase base, off_t upper_bound = -1);
+	std::string format_offset(BitOffset offset, OffsetBase base, BitOffset upper_bound = BitOffset::INVALID);
 	
 	template<typename T> typename T::iterator const_iterator_to_iterator(typename T::const_iterator &const_iter, T &container)
 	{
@@ -106,6 +131,58 @@ namespace REHex {
 	
 	std::string document_save_as_dialog(wxWindow *modal_parent, Document *document);
 	
+	struct CarryBits
+	{
+		unsigned char value;
+		unsigned char mask;
+		
+		CarryBits():
+			value(0),
+			mask(0) {}
+		
+		CarryBits(unsigned char value, unsigned char mask):
+			value(value),
+			mask(mask) {}
+	};
+	
+	/**
+	 * @brief Copy memory with left bit shifting.
+	 *
+	 * @param dst   Destination buffer.
+	 * @param src   Source buffer.
+	 * @param n     Number of bytes to copy.
+	 * @param shift Number of bits to left shift by (0-7).
+	 *
+	 * @returns The bits removed from the first byte.
+	 *
+	 * Copies a range of bytes between buffers, left shifting bits through the entire range,
+	 * removing the leftmost bits from the first byte and inserting zeros to the rightmost end
+	 * of the last byte.
+	 *
+	 * Any bits shifted off the end of the first byte are returned, shifted ready for being
+	 * bitwise OR'd into the end of a buffer preceeding dst when copying in chunks.
+	*/
+	CarryBits memcpy_left(void *dst, const void *src, size_t n, int shift);
+	
+	/**
+	 * @brief Copy memory with right bit shifting.
+	 *
+	 * @param dst   Destination buffer.
+	 * @param src   Source buffer.
+	 * @param n     Number of bytes to copy.
+	 * @param shift Number of bits to right shift by (0-7).
+	 *
+	 * @returns The surplus bits from the end of the last byte.
+	 *
+	 * Copies a range of bytes between buffers, right shifting bits through the entire range,
+	 * removing the rightmost bits from the last byte and preserving the existing bits to the
+	 * left of where the bits are placed in the destination buffer.
+	 *
+	 * Any bits shifted off the end of the last byte are returned, shifted ready for being
+	 * bitwise OR'd into the start of a buffer following dst when copying in chunks.
+	*/
+	CarryBits memcpy_right(void *dst, const void *src, size_t n, int shift);
+	
 	/**
 	 * @brief A wxColour that can be used as a key in a map/etc.
 	*/
@@ -149,22 +226,9 @@ namespace REHex {
 			}
 	};
 	
-	/**
-	 * @brief Adds two integers together, clamping to the range of the type.
-	 *
-	 * This function adds two integer-type values together, if the result would overflow or
-	 * underflow, the result is clamped to the maximum or minimum value representable by the
-	 * type T.
-	 *
-	 * If the "overflow" parameter is non-NULL, whether or not an overflow (or overflow) was
-	 * detected is stored there.
-	*/
-	template<typename T> T add_clamp_overflow(T a, T b, bool *overflow = NULL)
+	template<typename T> T _add_clamp_overflow(T a, T b, bool *overflow, T T_min, T T_max, T T_zero)
 	{
-		const T T_min = std::numeric_limits<T>::min();
-		const T T_max = std::numeric_limits<T>::max();
-		
-		if((a < 0) != (b < 0))
+		if((a < T_zero) != (b < T_zero))
 		{
 			/* a and b have differing signs - can't overflow */
 			if(overflow != NULL)
@@ -174,7 +238,7 @@ namespace REHex {
 			
 			return a + b;
 		}
-		else if(a < 0)
+		else if(a < T_zero)
 		{
 			/* a and b are negative */
 			
@@ -222,6 +286,32 @@ namespace REHex {
 			}
 		}
 	}
+	
+	/**
+	 * @brief Adds two integers together, clamping to the range of the type.
+	 *
+	 * This function adds two integer-type values together, if the result would overflow or
+	 * underflow, the result is clamped to the maximum or minimum value representable by the
+	 * type T.
+	 *
+	 * If the "overflow" parameter is non-NULL, whether or not an overflow (or underflow) was
+	 * detected is stored there.
+	*/
+	template<typename T> T add_clamp_overflow(T a, T b, bool *overflow = NULL)
+	{
+		return _add_clamp_overflow<T>(a, b, overflow, std::numeric_limits<T>::min(), std::numeric_limits<T>::max(), 0);
+	}
+	
+	/**
+	 * @brief Specialisation of add_clamp_overflow<T>() for BitOffset.
+	*/
+	template<> BitOffset add_clamp_overflow(BitOffset a, BitOffset b, bool *overflow);
+	
+	json_t *colour_to_json(const wxColour &colour);
+	wxColour colour_from_json(const json_t *json);
+	
+	std::string colour_to_string(const wxColour &colour);
+	wxColour colour_from_string(const std::string &s);
 }
 
 #endif /* !REHEX_UTIL_HPP */
